@@ -55,13 +55,16 @@ def cli(ctx, **kwargs):
 
     config = Config()
 
+    if ctx.invoked_subcommand == "config":
+        return
+
     if config.session["check_for_updates"]:
         from importlib import metadata
 
         from packaging import version
 
         r = requests.get("https://pypi.org/pypi/streamrip/json").json()
-        newest = r['info']['version']
+        newest = r["info"]["version"]
         if version.parse(metadata.version("streamrip")) < version.parse(newest):
             click.secho(
                 "A new version of streamrip is available! "
@@ -69,7 +72,7 @@ def cli(ctx, **kwargs):
                 fg="yellow",
             )
         else:
-            click.secho("streamrip is up-to-date!", fg='green')
+            click.secho("streamrip is up-to-date!", fg="green")
 
     if kwargs["no_db"]:
         config.session["database"]["enabled"] = False
@@ -216,6 +219,33 @@ def discover(ctx, **kwargs):
         none_chosen()
 
 
+@cli.command()
+@click.option(
+    "-s", "--source", help="Qobuz, Tidal, Deezer, or SoundCloud. Default: Qobuz."
+)
+@click.argument("URL")
+@click.pass_context
+def lastfm(ctx, source, url):
+    """Searches for tracks from a last.fm playlist on a given source.
+
+    Examples:
+
+        $ rip lastfm https://www.last.fm/user/nathan3895/playlists/12059037
+
+        Download a playlist using Qobuz as the source
+
+        $ rip lastfm -s tidal https://www.last.fm/user/nathan3895/playlists/12059037
+
+        Download a playlist using Tidal as the source
+    """
+
+    if source is not None:
+        config.session["lastfm"]["source"] = source
+
+    core.handle_lastfm_urls(url)
+    core.download()
+
+
 @cli.command()
 @click.option("-o", "--open", is_flag=True, help="Open the config file")
 @click.option("-q", "--qobuz", is_flag=True, help="Set Qobuz credentials")
