@@ -26,7 +26,7 @@ from .exceptions import (
     InvalidQuality,
 )
 from .spoofbuz import Spoofer
-from .utils import get_quality
+from .utils import gen_threadsafe_session, get_quality
 
 urllib3.disable_warnings()
 requests.adapters.DEFAULT_RETRIES = 5
@@ -149,12 +149,8 @@ class QobuzClient(ClientInterface):
         self.app_id = str(kwargs["app_id"])  # Ensure it is a string
         self.secrets = kwargs["secrets"]
 
-        self.session = requests.Session()
-        self.session.headers.update(
-            {
-                "User-Agent": AGENT,
-                "X-App-Id": self.app_id,
-            }
+        self.session = gen_threadsafe_session(
+            headers={"User-Agent": AGENT, "X-App-Id": self.app_id}
         )
 
         self._api_login(email, pwd)
@@ -373,7 +369,9 @@ class DeezerClient(ClientInterface):
     max_quality = 2
 
     def __init__(self):
-        self.session = requests.Session()
+        self.session = gen_threadsafe_session()
+
+        # no login required
         self.logged_in = True
 
     def search(self, query: str, media_type: str = "album", limit: int = 200) -> dict:
@@ -389,9 +387,6 @@ class DeezerClient(ClientInterface):
         # TODO: more robust url sanitize
         query = query.replace(" ", "+")
 
-        if media_type.endswith("s"):
-            media_type = media_type[:-1]
-
         # TODO: use limit parameter
         response = self.session.get(f"{DEEZER_BASE}/search/{media_type}?q={query}")
         response.raise_for_status()
@@ -447,6 +442,8 @@ class TidalClient(ClientInterface):
         self.refresh_token = None
         self.expiry = None
 
+        self.session = gen_threadsafe_session()
+
     def login(
         self,
         user_id=None,
@@ -489,7 +486,11 @@ class TidalClient(ClientInterface):
             "assetpresentation": "FULL",
         }
         resp = self._api_request(f"tracks/{track_id}/playbackinfopostpaywall", params)
-        manifest = json.loads(base64.b64decode(resp["manifest"]).decode("utf-8"))
+        try:
+            manifest = json.loads(base64.b64decode(resp["manifest"]).decode("utf-8"))
+        except KeyError:
+            raise Exception(resp["userMessage"])
+
         logger.debug(manifest)
         return {
             "url": manifest["urls"][0],
@@ -584,7 +585,9 @@ class TidalClient(ClientInterface):
         headers = {
             "authorization": f"Bearer {token}",
         }
-        r = requests.get("https://api.tidal.com/v1/sessions", headers=headers).json()
+        r = self.session.get(
+            "https://api.tidal.com/v1/sessions", headers=headers
+        ).json()
         if r.status != 200:
             raise Exception("Login failed")
 
@@ -610,10 +613,13 @@ class TidalClient(ClientInterface):
         self.country_code = resp["user"]["countryCode"]
         self.access_token = resp["access_token"]
         self.token_expiry = resp["expires_in"] + time.time()
+        self._update_authorization()
 
     def _login_by_access_token(self, token, user_id=None):
-        headers = {"authorization": f"Bearer {token}"}
-        resp = requests.get("https://api.tidal.com/v1/sessions", headers=headers).json()
+        headers = {"authorization": f"Bearer {token}"}  # temporary
+        resp = self.session.get(
+            "https://api.tidal.com/v1/sessions", headers=headers
+        ).json()
         if resp.get("status", 200) != 200:
             raise Exception(f"Login failed {resp}")
 
@@ -623,6 +629,7 @@ class TidalClient(ClientInterface):
         self.user_id = resp["userId"]
         self.country_code = resp["countryCode"]
         self.access_token = token
+        self._update_authorization()
 
     def _api_get(self, item_id: str, media_type: str) -> dict:
         url = f"{media_type}s/{item_id}"
@@ -650,22 +657,27 @@ class TidalClient(ClientInterface):
         if params is None:
             params = {}
 
-        headers = {"authorization": f"Bearer {self.access_token}"}
         params["countryCode"] = self.country_code
         params["limit"] = 100
-        r = requests.get(f"{TIDAL_BASE}/{path}", headers=headers, params=params).json()
+        r = self.session.get(f"{TIDAL_BASE}/{path}", params=params).json()
         return r
 
     def _api_post(self, url, data, auth=None):
-        r = requests.post(url, data=data, auth=auth, verify=False).json()
+        r = self.session.post(url, data=data, auth=auth, verify=False).json()
         return r
 
+    def _update_authorization(self):
+        self.session.headers.update({"authorization": f"Bearer {self.access_token}"})
+
 
 class SoundCloudClient(ClientInterface):
     source = "soundcloud"
     max_quality = 0
     logged_in = True
 
+    def __init__(self):
+        self.session = gen_threadsafe_session(headers={"User-Agent": AGENT})
+
     def login(self):
         raise NotImplementedError
 
@@ -717,7 +729,7 @@ class SoundCloudClient(ClientInterface):
             url = f"{SOUNDCLOUD_BASE}/{path}"
 
         logger.debug(f"Fetching url {url}")
-        r = requests.get(url, params=params)
+        r = self.session.get(url, params=params)
         if resp_obj:
             return r
 
