@@ -18,13 +18,6 @@ AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:83.0) Gecko/20100101 Firef
 
 TIDAL_COVER_URL = "https://resources.tidal.com/images/{uuid}/{width}x{height}.jpg"
 
-EXT = {
-    0: ".mp3",
-    1: ".mp3",
-    2: ".flac",
-    3: ".flac",
-    4: ".flac",
-}
 
 QUALITY_DESC = {
     0: "128kbps",
@@ -141,11 +134,13 @@ FOLDER_FORMAT = (
 TRACK_FORMAT = "{tracknumber}. {artist} - {title}"
 
 URL_REGEX = (
-    r"https:\/\/(?:www|open|play|listen)?\.?(qobuz|tidal|deezer)\.com(?:(?:\/(track|playlist|album|"
-    r"artist|label))|(?:\/[-\w]+?))+\/([-\w]+)"
+    r"https?://(?:www|open|play|listen)?\.?(qobuz|tidal|deezer)\.com(?:(?:/"
+    r"(track|playlist|album|artist|label))|(?:\/[-\w]+?))+\/([-\w]+)"
 )
+
 SOUNDCLOUD_URL_REGEX = r"https://soundcloud.com/[-\w:/]+"
 SOUNDCLOUD_CLIENT_ID = "a3e059563d7fd3372b49b37f00a00bcf"
+LASTFM_URL_REGEX = r"https://www.last.fm/user/\w+/playlists/\w+"
 
 
 TIDAL_MAX_Q = 7
