@@ -2,25 +2,34 @@ import logging
 import os
 import re
 import sys
+import threading
 from getpass import getpass
 from hashlib import md5
 from string import Formatter
 from typing import Generator, Optional, Tuple, Union
 
 import click
+import requests
+from tqdm import tqdm
 
 from .clients import DeezerClient, QobuzClient, SoundCloudClient, TidalClient
 from .config import Config
 from .constants import (
     CONFIG_PATH,
     DB_PATH,
+    LASTFM_URL_REGEX,
     MEDIA_TYPES,
     SOUNDCLOUD_URL_REGEX,
     URL_REGEX,
 )
 from .db import MusicDB
-from .downloader import Album, Artist, Label, Playlist, Track
-from .exceptions import AuthenticationError, ParsingError
+from .downloader import Album, Artist, Label, Playlist, Track, Tracklist
+from .exceptions import (
+    AuthenticationError,
+    NonStreamable,
+    NoResultsFound,
+    ParsingError,
+)
 from .utils import capitalize
 
 logger = logging.getLogger(__name__)
@@ -44,6 +53,8 @@ class MusicDL(list):
 
         self.url_parse = re.compile(URL_REGEX)
         self.soundcloud_url_parse = re.compile(SOUNDCLOUD_URL_REGEX)
+        self.lastfm_url_parse = re.compile(LASTFM_URL_REGEX)
+
         self.config = config
         if self.config is None:
             self.config = Config(CONFIG_PATH)
@@ -108,13 +119,14 @@ class MusicDL(list):
             self.prompt_creds(source)
 
     def handle_urls(self, url: str):
-        """Download an url
+        """Download a url
 
         :param url:
         :type url: str
         :raises InvalidSourceError
         :raises ParsingError
         """
+
         for source, url_type, item_id in self.parse_urls(url):
             if item_id in self.db:
                 logger.info(
@@ -142,27 +154,45 @@ class MusicDL(list):
         self.append(item)
 
     def download(self):
+        try:
+            arguments = {
+                "database": self.db,
+                "parent_folder": self.config.session["downloads"]["folder"],
+                "folder_format": self.config.session["path_format"]["folder"],
+                "track_format": self.config.session["path_format"]["track"],
+                "embed_cover": self.config.session["artwork"]["embed"],
+                "embed_cover_size": self.config.session["artwork"]["size"],
+                "keep_hires_cover": self.config.session["artwork"]["keep_hires_cover"],
+                "set_playlist_to_album": self.config.session["metadata"][
+                    "set_playlist_to_album"
+                ],
+                "stay_temp": self.config.session["conversion"]["enabled"],
+                "conversion": self.config.session["conversion"],
+                "concurrent_downloads": self.config.session["concurrent_downloads"],
+                "new_tracknumbers": self.config.session["metadata"][
+                    "new_playlist_tracknumbers"
+                ],
+            }
+        except KeyError as err:
+            click.secho(
+                "There was a problem with your config file. This happens "
+                "sometimes after updates. Run ",
+                nl=False,
+                fg="red",
+            )
+            click.secho("rip config --reset ", fg="yellow", nl=False)
+            click.secho("to reset it. You will need to log in again.", fg="red")
+            logger.debug(err)
+            exit()
 
-        arguments = {
-            "database": self.db,
-            "parent_folder": self.config.session["downloads"]["folder"],
-            "folder_format": self.config.session["path_format"]["folder"],
-            "track_format": self.config.session["path_format"]["track"],
-            "keep_downloaded_cover": self.config.session["artwork"][
-                "keep_downloaded_cover"
-            ],
-            "keep_embedded_cover": self.config.session["artwork"][
-                "keep_embedded_cover"
-            ],
-            "embed_cover": self.config.session["artwork"]["embed"],
-            "embed_cover_size": self.config.session["artwork"]["embed_size"],
-            "download_cover_size": self.config.session["artwork"]["download_size"],
-        }
         logger.debug("Arguments from config: %s", arguments)
+
+        source_subdirs = self.config.session["downloads"]["source_subdirectories"]
         for item in self:
-            if self.config.session["downloads"]["source_subdirectories"]:
-                arguments["parent_folder"] = os.path.join(
-                    arguments["parent_folder"], capitalize(item.client.source)
+
+            if source_subdirs:
+                arguments["parent_folder"] = self.__get_source_subdir(
+                    item.client.source
                 )
 
             arguments["quality"] = self.config.session[item.client.source]["quality"]
@@ -173,7 +203,12 @@ class MusicDL(list):
                 arguments["filters"] = filters_
                 logger.debug("Added filter argument for artist/label: %s", filters_)
 
-            item.load_meta()
+            if not (isinstance(item, Tracklist) and item.loaded):
+                try:
+                    item.load_meta()
+                except NonStreamable:
+                    click.secho(f"{item!s} is not available, skipping.", fg="red")
+                    continue
 
             if isinstance(item, Track):
                 # track.download doesn't automatically tag
@@ -181,11 +216,11 @@ class MusicDL(list):
             else:
                 item.download(**arguments)
 
-            if self.db != []:
+            if self.db != [] and hasattr(item, "id"):
                 self.db.add(item.id)
 
-            if self.config.session["conversion"]["enabled"]:
-                item.convert(**self.config.session["conversion"])
+            # if self.config.session["conversion"]["enabled"]:
+            # item.convert(**self.config.session["conversion"])
 
     def get_client(self, source: str):
         client = self.clients[source]
@@ -248,6 +283,42 @@ class MusicDL(list):
 
         raise ParsingError(f"Error parsing URL: `{url}`")
 
+    def handle_lastfm_urls(self, urls):
+        lastfm_urls = self.lastfm_url_parse.findall(urls)
+        lastfm_source = self.config.session["lastfm"]["source"]
+        tracks_not_found = 0
+
+        def search_query(query: str, playlist: Playlist):
+            global tracks_not_found
+            try:
+                track = next(self.search(lastfm_source, query, media_type="track"))
+                playlist.append(track)
+            except NoResultsFound:
+                tracks_not_found += 1
+                return
+
+        for purl in lastfm_urls:
+            click.secho(f"Fetching playlist at {purl}", fg="blue")
+            title, queries = self.get_lastfm_playlist(purl)
+
+            pl = Playlist(client=self.get_client(lastfm_source), name=title)
+            processes = []
+
+            for title, artist in queries:
+                query = f"{title} {artist}"
+                proc = threading.Thread(
+                    target=search_query, args=(query, pl), daemon=True
+                )
+                proc.start()
+                processes.append(proc)
+
+            for proc in tqdm(processes, unit="tracks", desc="Searching"):
+                proc.join()
+
+            pl.loaded = True
+            click.secho(f"{tracks_not_found} tracks not found.", fg="yellow")
+            self.append(pl)
+
     def handle_txt(self, filepath: Union[str, os.PathLike]):
         """
         Handle a text file containing URLs. Lines starting with `#` are ignored.
@@ -282,9 +353,13 @@ class MusicDL(list):
                     if i > limit:
                         return
         else:
-            for item in (
+            items = (
                 results.get("data") or results.get("items") or results.get("collection")
-            ):
+            )
+            if items is None:
+                raise NoResultsFound(query)
+
+            for item in items:
                 yield MEDIA_CLASS[media_type].from_api(item, client)
                 i += 1
                 if i > limit:
@@ -296,7 +371,8 @@ class MusicDL(list):
                 "{albumartist} - {title}\n"
                 "Released on {year}\n{tracktotal} tracks\n"
                 "{bit_depth} bit / {sampling_rate} Hz\n"
-                "Version: {version}"
+                "Version: {version}\n"
+                "Genre: {genre}"
             )
         elif isinstance(media, Artist):
             fmt = "{name}"
@@ -392,3 +468,40 @@ class MusicDL(list):
                     for i in choice:
                         self.append(results[i])
                 return True
+
+    def get_lastfm_playlist(self, url: str) -> Tuple[str, list]:
+        info = []
+        words = re.compile(r"[\w\s]+")
+        title_tags = re.compile('title="([^"]+)"')
+
+        def essence(s):
+            s = re.sub(r"&#\d+;", "", s)  # remove HTML entities
+            return "".join(words.findall(s))
+
+        def get_titles(s):
+            titles = title_tags.findall(s)[2:]
+            for i in range(0, len(titles) - 1, 2):
+                info.append((essence(titles[i]), essence(titles[i + 1])))
+
+        r = requests.get(url)
+        get_titles(r.text)
+        remaining_tracks = (
+            int(re.search(r'data-playlisting-entry-count="(\d+)"', r.text).group(1))
+            - 50
+        )
+        playlist_title = re.search(
+            r'<h1 class="playlisting-playlist-header-title">([^<]+)</h1>', r.text
+        ).group(1)
+
+        page = 1
+        while remaining_tracks > 0:
+            page += 1
+            r = requests.get(f"{url}?page={page}")
+            get_titles(r.text)
+            remaining_tracks -= 50
+
+        return playlist_title, info
+
+    def __get_source_subdir(self, source: str) -> str:
+        path = self.config.session["downloads"]["folder"]
+        return os.path.join(path, capitalize(source))
