@@ -1,8 +1,8 @@
+"""Manages the information that will be embeded in the audio file. """
+
 import json
 import logging
 import re
-import sys
-from pprint import pprint
 from typing import Generator, Optional, Tuple, Union
 
 from .constants import (
@@ -14,6 +14,7 @@ from .constants import (
     TRACK_KEYS,
 )
 from .exceptions import InvalidContainerError
+from .utils import safe_get
 
 logger = logging.getLogger(__name__)
 
@@ -41,6 +42,7 @@ class TrackMetadata:
         * tracknumber
         * discnumber
         * tracktotal
+        * disctotal
 
     """
 
@@ -55,6 +57,7 @@ class TrackMetadata:
         :param album: album dict from API
         :type album: Optional[dict]
         """
+        self.title = None
         self.album = None
         self.albumartist = None
         self.composer = None
@@ -68,8 +71,12 @@ class TrackMetadata:
         self.cover = None
         self.tracknumber = None
         self.discnumber = None
+        self.disctotal = None
 
-        self.__source = source  # not included in tags
+        # not included in tags
+        self.explicit = False
+
+        self.__source = source
 
         if track is None and album is None:
             logger.debug("No params passed, returning")
@@ -89,16 +96,24 @@ class TrackMetadata:
         """
         if self.__source == "qobuz":
             self.album = resp.get("title")
-            self.tracktotal = str(resp.get("tracks_count", 1))
+            self.tracktotal = resp.get("tracks_count", 1)
             self.genre = resp.get("genres_list", [])
             self.date = resp.get("release_date_original") or resp.get("release_date")
+            if self.date:
+                self.year = self.date[:4]
+
             self.copyright = resp.get("copyright")
-            self.albumartist = resp.get("artist", {}).get("name")
+            self.albumartist = safe_get(resp, "artist", "name")
             self.label = resp.get("label")
             self.description = resp.get("description")
-            self.disctotal = max(
-                track.get("media_number", 1) for track in resp["tracks"]["items"]
+            self.disctotal = (
+                max(
+                    track.get("media_number", 1)
+                    for track in safe_get(resp, "tracks", "items", default=[{}])
+                )
+                or 1
             )
+            self.explicit = resp.get("parental_warning", False)
 
             if isinstance(self.label, dict):
                 self.label = self.label.get("name")
@@ -108,27 +123,34 @@ class TrackMetadata:
             self.tracktotal = resp.get("numberOfTracks")
             # genre not returned by API
             self.date = resp.get("releaseDate")
+            if self.date:
+                self.year = self.date[:4]
+
             self.copyright = resp.get("copyright")
-            self.albumartist = resp.get("artist", {}).get("name")
+            self.albumartist = safe_get(resp, "artist", "name")
             self.disctotal = resp.get("numberOfVolumes")
             self.isrc = resp.get("isrc")
+            self.explicit = resp.get("explicit", False)
             # label not returned by API
 
         elif self.__source == "deezer":
             self.album = resp.get("title")
             self.tracktotal = resp.get("track_total")
-            self.genre = resp.get("genres", {}).get("data")
+            self.genre = safe_get(resp, "genres", "data")
             self.date = resp.get("release_date")
-            self.albumartist = resp.get("artist", {}).get("name")
+            self.albumartist = safe_get(resp, "artist", "name")
             self.label = resp.get("label")
+            # either 0 or 1
+            self.explicit = bool(resp.get("parental_warning"))
+
         elif self.__source == "soundcloud":
             raise NotImplementedError
         else:
             raise ValueError(self.__source)
 
     def add_track_meta(self, track: dict):
-        """Parse the metadata from a track dict returned by the
-        Qobuz API.
+        """Parse the metadata from a track dict returned by an
+        API.
 
         :param track:
         """
@@ -137,25 +159,23 @@ class TrackMetadata:
             self._mod_title(track.get("version"), track.get("work"))
             self.composer = track.get("composer", {}).get("name")
 
-            self.tracknumber = f"{int(track.get('track_number', 1)):02}"
-            self.discnumber = str(track.get("media_number", 1))
-            try:
-                self.artist = track["performer"]["name"]
-            except KeyError:
-                if hasattr(self, "albumartist"):
-                    self.artist = self.albumartist
+            self.tracknumber = track.get("track_number", 1)
+            self.discnumber = track.get("media_number", 1)
+            self.artist = safe_get(track, "performer", "name")
+            if self.artist is None:
+                self.artist = self.get("albumartist")
 
         elif self.__source == "tidal":
             self.title = track.get("title").strip()
             self._mod_title(track.get("version"), None)
-            self.tracknumber = f"{int(track.get('trackNumber', 1)):02}"
-            self.discnumber = str(track.get("volumeNumber"))
+            self.tracknumber = track.get("trackNumber", 1)
+            self.discnumber = track.get("volumeNumber")
             self.artist = track.get("artist", {}).get("name")
 
         elif self.__source == "deezer":
             self.title = track.get("title").strip()
             self._mod_title(track.get("version"), None)
-            self.tracknumber = f"{int(track.get('track_position', 1)):02}"
+            self.tracknumber = track.get("track_position", 1)
             self.discnumber = track.get("disk_number")
             self.artist = track.get("artist", {}).get("name")
 
@@ -184,7 +204,7 @@ class TrackMetadata:
             self.title = f"{work}: {self.title}"
 
     @property
-    def artist(self) -> Union[str, None]:
+    def artist(self) -> Optional[str]:
         """Returns the value to set for the artist tag. Defaults to
         `self.albumartist` if there is no track artist.
 
@@ -196,6 +216,8 @@ class TrackMetadata:
         if self._artist is not None:
             return self._artist
 
+        return None
+
     @artist.setter
     def artist(self, val: str):
         """Sets the internal artist variable to val.
@@ -220,8 +242,13 @@ class TrackMetadata:
         if isinstance(self._genres, list):
             genres = re.findall(r"([^\u2192\/]+)", "/".join(self._genres))
             no_repeats = []
-            [no_repeats.append(g) for g in genres if g not in no_repeats]
+
+            for genre in genres:
+                if genre not in no_repeats:
+                    no_repeats.append(genre)
+
             return ", ".join(no_repeats)
+
         elif isinstance(self._genres, str):
             return self._genres
 
@@ -247,9 +274,9 @@ class TrackMetadata:
         if hasattr(self, "_copyright"):
             if self._copyright is None:
                 return None
-            cr = self._copyright.replace("(P)", PHON_COPYRIGHT)
-            cr = cr.replace("(C)", COPYRIGHT)
-            return cr
+            copyright = re.sub(r"(?i)\(P\)", PHON_COPYRIGHT, self._copyright)
+            copyright = re.sub(r"(?i)\(C\)", COPYRIGHT, copyright)
+            return copyright
 
         logger.debug("Accessed copyright tag before setting, return None")
         return None
@@ -265,7 +292,7 @@ class TrackMetadata:
         self._copyright = val
 
     @property
-    def year(self) -> Union[str, None]:
+    def year(self) -> Optional[str]:
         """Returns the year published of the track.
 
         :rtype: str
@@ -277,6 +304,8 @@ class TrackMetadata:
             if self.date is not None:
                 return self.date[:4]
 
+        return None
+
     @year.setter
     def year(self, val):
         """Sets the internal year variable to val.
@@ -317,12 +346,12 @@ class TrackMetadata:
         container = container.lower()
         if container in ("flac", "vorbis"):
             return self.__gen_flac_tags()
-        elif container in ("mp3", "id3"):
+        if container in ("mp3", "id3"):
             return self.__gen_mp3_tags()
-        elif container in ("alac", "m4a", "mp4", "aac"):
+        if container in ("alac", "m4a", "mp4", "aac"):
             return self.__gen_mp4_tags()
-        else:
-            raise InvalidContainerError(f"Invalid container {container}")
+
+        raise InvalidContainerError(f"Invalid container {container}")
 
     def __gen_flac_tags(self) -> Tuple[str, str]:
         """Generate key, value pairs to tag FLAC files.
@@ -332,7 +361,10 @@ class TrackMetadata:
         for k, v in FLAC_KEY.items():
             tag = getattr(self, k)
             if tag:
-                logger.debug(f"Adding tag {v}: {repr(tag)}")
+                if k in ("tracknumber", "discnumber", "tracktotal", "disctotal"):
+                    tag = f"{int(tag):02}"
+
+                logger.debug("Adding tag %s: %s", v, tag)
                 yield (v, str(tag))
 
     def __gen_mp3_tags(self) -> Tuple[str, str]:
@@ -351,14 +383,22 @@ class TrackMetadata:
             if text is not None and v is not None:
                 yield (v.__name__, v(encoding=3, text=text))
 
-    def __mp4_tags(self) -> Tuple[str, str]:
+    def __gen_mp4_tags(self) -> Tuple[str, Union[str, int, tuple]]:
         """Generate key, value pairs to tag ALAC or AAC files in
         an MP4 container.
 
         :rtype: Tuple[str, str]
         """
         for k, v in MP4_KEY.items():
-            return (v, getattr(self, k))
+            if k == "tracknumber":
+                text = [(self.tracknumber, self.tracktotal)]
+            elif k == "discnumber":
+                text = [(self.discnumber, self.get("disctotal", 1))]
+            else:
+                text = getattr(self, k)
+
+            if v is not None and text is not None:
+                yield (v, text)
 
     def __setitem__(self, key, val):
         """Dict-like access for tags.
